/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
 *  - http://212.187.12.197/RNG/terraform/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdio.h>
#include <string.h>
#include <gtk--/accelgroup.h>
#include <gtk--/main.h>
#include <gtk--/radiomenuitem.h>
#include <gtk--/style.h>

#include "TFWindow.h"
#include "HeightField.h"
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"
#include "MenuDefs.h"
#include "TFOptions.h"


// base color for each color band 
static char	*bandBaseCol[]=	
			{	
				"MidnightBlue",
				"MediumBlue",
				"Blue",
				"Dodger Blue",
				"Green2",
				"Green3",
				"Green4",
				"DarkOliveGreen4",
				"LightGoldenrod2",
				"LightGoldenrod3",
				"LightGoldenrod4",
				"LightYellow4",
				"DimGray",
				"Gray",
				"LightGray",	};

static char 	*bandBaseCol_Desert[]=
			{
				"LightGoldenrod3",
				"LightGoldenrod4",
				"DimGray",
				"LightSlateGray",
				"Gray",
				"LightGray",	};


// never acutally use/compile this code
#if 0
	// This is a dummy object to mislead xgettext into properly 
	// extracting the translatable menu strings. xgettext unfortunately
	// doesn't grok macros, so we need this table to fool it. 
	// This must be kept in sync with MenuDefs.h 
	// The following script will generate this table for you:
	/*
	cat MenuDefs.h | grep "\"" |
	awk ' BEGIN	{ FS = "\""; print ("char *foobar[] = {"); }
			{ printf ("\t\t\t\tN_(\"%s\"),\n", $2); }
		 END	{ print (" };"); } '
	*/
char *foobar[] = {
				N_("File"),
				N_("_File"),
				N_("New"),
				N_("New/"),
				N_("Spectral S_ynthesis"),
				N_("Sub_division"),
				N_("Ra_ndom"),
				N_("_Open"),
				N_("_Merge"),
				N_("Reload"),
				N_("_Save"),
				N_("Save _As ..."),
				N_("Clos_e"),
				N_("Export"),
				N_("Gimp"),
				N_("Open_GL"),
				N_("PO_VRay"),
				N_("O_ptions"),
				N_("Print Settings"),
				N_("About"),
				N_("_Quit"),
				N_("HF"),
				N_("_HF"),
				N_("_Craters"),
				N_("Fill"),
				N_("_Fold"),
				N_("Gaussian _Hill"),
				N_("_Linear Scale"),
				N_("Mirror"),
				N_("Rotate"),
				N_("Ro_ughen&Smooth"),
				N_("Terrace"),
				N_("_Transform"),
				N_("Clone"),
				N_("Contour Line Map"),
				N_("Erode"),
				N_("Flowmap"),
				N_("_Invert"),
				N_("Place POV Object"),
				N_("Rescale"),
				N_("_Render Options"),
				N_("View"),
				N_("_View"),
				N_("2D Plane"),
				N_("3D Wire"),
				N_("3D Height"),
				N_("3D Light"),
				N_("Fast Wireframe"),
				N_("Auto Rotate"),
				N_("_Colormap"),
				N_("Color Bands"),
				N_("Desert"),
				N_("Grayscale"),
				N_("RedHot"),
				N_("Hide"),
				N_("Show"),
				N_("Clone"),
				N_("Close"),
 };
#endif


guint TFWhash (gconstpointer key);			// return hash for key
gint TFWhashcmp (gconstpointer a, gconstpointer b);	// compare 

 
/*
 *  constructor: build the window with an itemfactory, buffered drawing area 
 * 	and colormaps. Store the generated menu in a hash table so we have 
 *	a handy reference later we can use for set_sensitvity calls
 */
TFWindow::TFWindow() : GuiBufferedDrawingArea (),
       			d_vbox (FALSE, 0)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFWindow\n");

	p_mNames = NULL;
	p_linearColorMap = p_linearColorMap2 = NULL;
	p_bandColorMap = p_bandColorMap2 = NULL;

	p_menuBar = manage (new Gtk::MenuBar());
	p_win = NULL;
	p_popupMenu = NULL;

	p_miReload = p_miMerge = NULL;

	if (TFOptions::s_fixedMenus)
		buildWindowFixedMenu ();
	else
		buildWindowFloatingMenu ();
}


/*
 *  destructor: clean up 
 */
TFWindow::~TFWindow()
{
	this->hide ();
	p_win->hide ();

	delete p_win;
	delete p_linearColorMap;
	delete p_bandColorMap;
	delete [] p_mNames;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFWindow\n");
}


/*
 *  quit: exit
 */
void TFWindow::quit ()
{
	gtk_exit (0);
}


/*
 *  hide_impl: hide everything there is to hide
 */
//void TFWindow::hide_impl ()
//{
//	p_win->hide ();
//}


/*
 *  show_impl: show everything there is to show 
 */
//void TFWindow::show_impl ()
//{
//	this->realize ();
//	//p_win->show ();
//}


/*
 *  buildWindowFixedMenu: build a TFWindow with the menu contained in a menubar.
 */
void TFWindow::buildWindowFixedMenu ()
{
	Gtk::Style 	*s=Gtk::Style::create();  
	s->set_font(Gdk_Font("-*-Helvetica-Medium-R-Normal--*-100-*-*-*-*-*-*"));

	p_win = new Gtk::Window (GTK_WINDOW_TOPLEVEL);
	p_win->set_style (*s);
	p_win->add (d_vbox);

	addMenus ();

	d_vbox.pack_start (*p_menuBar, FALSE, FALSE, 0);

	// create and attach drawing area 
	d_vbox.pack_start (*this, TRUE, TRUE, 0);

	allocColormaps ();

	// catch destroy event
	destroy.connect (SigC::slot(this,&TFWindow::quit));


	p_win->show_all ();
	p_win->set_policy (FALSE, FALSE, FALSE);
	d_vbox.show ();
	p_menuBar->show ();
	this->show ();

	// since Gtk-- 0.11.0 this has to be done here in order to properly 
	// size the window when staring without loading a height field. 
	//this->set_usize (400, 400);
	this->setSize (400, 400);
	this->show_all ();
}


/*
 *  buildWindowFloatingMenu: build a TFWindow with a popup menu
 */
void TFWindow::buildWindowFloatingMenu ()
{
	Gtk::Style 	*s=Gtk::Style::create();  
	s->set_font(Gdk_Font("-*-Helvetica-Medium-R-Normal--*-100-*-*-*-*-*-*"));

	p_win = new Gtk::Window (GTK_WINDOW_TOPLEVEL);
	p_win->set_style (*s);
	p_win->add (d_vbox);

	addMenus ();

	// create and attach drawing area 
	d_vbox.pack_start (*this, TRUE, TRUE, 0);

	allocColormaps ();

	// catch destroy event
	destroy.connect (SigC::slot(this,&TFWindow::quit));

	p_win->show_all ();
	p_win->set_policy (FALSE, FALSE, FALSE);
	d_vbox.show ();
	this->show ();

	// since Gtk-- 0.11.0 this has to be done here in order to properly 
	// size the window when staring without loading a height field. 
	//this->set_usize (400, 400);
	this->setSize (400, 400);
	this->show_all ();
}


/*
 *  addMenus: build up the itemFactory by adding the individual menus
 */
void TFWindow::addMenus ()
{
	char				*s, *t;
	string				defaultKey (_(MENU_VIEW_3DWIRE));
	Gtk::CheckMenuItem		defaultView;
	Gtk::AccelGroup			*accgrp = Gtk::AccelGroup::create();
	Gtk::Menu			*mFile = manage (new Gtk::Menu());
	Gtk::Menu			*mNew = manage (new Gtk::Menu());
	Gtk::Menu			*mExport = manage (new Gtk::Menu());
	Gtk::Menu			*mHF = manage (new Gtk::Menu());
	Gtk::Menu			*mView = manage (new Gtk::Menu());
	Gtk::Menu			*mCMap = manage (new Gtk::Menu());

	accgrp = Gtk::AccelGroup::create();

	t = new char[80];
	//p_menuHash = new Glib_Hash (TFWhash, TFWhashcmp);
	p_mNames = new char*[50];
	d_nameCount = 0;

	{
	using namespace Gtk::Menu_Helpers; 

	MenuList mlFile		= mFile->items();
	MenuList mlNew		= mNew->items();
	MenuList mlExport	= mExport->items();
	MenuList mlHF		= mHF->items();
	MenuList mlView		= mView->items();
	MenuList mlCMap		= mCMap->items();

#define ACC_C	TFWindow::buildAccel(s, ACCEL_TYPE_CNTRL, t)
#define ACC_A	TFWindow::buildAccel(s, ACCEL_TYPE_ALT, t)


	// 
	// Build New Submenu
	//
	s = _(MENU_FILE_NEW_BRANCH);
	MenuElem	newBranch (s, ACC_A, *mNew);		// New

	s = _(MENU_FILE_NEW_SYNTHESIS);
	MenuElem	nSpectral (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlNew.push_back (nSpectral);

	s = _(MENU_FILE_NEW_SUBDIVIDE);
	MenuElem	nSubdiv (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlNew.push_back (nSubdiv);

	s = _(MENU_FILE_NEW_RANDOM);
	MenuElem	nRandom (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlNew.push_back (nRandom);

	mlFile.push_back (newBranch);		// register submenu


	//
	// Build Export Submenu
	//
	s = _(MENU_FILE_EXPORT_BRANCH);
	MenuElem	exportBranch (s, ACC_A, *mExport);	// Export

	s = _(MENU_FILE_EXPORT_GIMP);
	MenuElem	eGimp (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlExport.push_back (eGimp);

#ifdef HAVE_OPENGL
	s = _(MENU_FILE_EXPORT_OPENGL);
	MenuElem	eOpenGL (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlExport.push_back (eOpenGL);
#endif

	s = _(MENU_FILE_EXPORT_POVRAY);
	MenuElem	ePovray (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlExport.push_back (ePovray);


	// 
	// Build File menu
	//
	// NEW ELEMENTS
	s=_(MENU_FILE_OPEN);
	MenuElem	fOpen (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlFile.push_back (fOpen);

	s=_(MENU_FILE_MERGE);
	MenuElem	fMerge (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlFile.push_back (fMerge);
	p_miMerge = *fMerge;

	s=_(MENU_FILE_RELOAD);
	MenuElem	fReload (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlFile.push_back (fReload);
	p_miReload = *fReload; 

	s=_(MENU_FILE_SAVE);
	MenuElem	fSave (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlFile.push_back (fSave);

	s=_(MENU_FILE_SAVEAS);
	MenuElem	fSaveAs (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlFile.push_back (fSaveAs);

	s=_(MENU_FILE_CLOSE);
	MenuElem	fClose (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlFile.push_back (fClose);
	mlFile.push_back (SeparatorElem());

	// EXPORT ELEMENTS
	mlFile.push_back (exportBranch);		// register submenu

	mlFile.push_back (SeparatorElem());

	s=_(MENU_FILE_OPTIONS);
	MenuElem	fOptions (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlFile.push_back (fOptions);

	s=_(MENU_FILE_PRINT_SETTINGS);
	MenuElem	fPrintSettings (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlFile.push_back (fPrintSettings);

	s=_(MENU_FILE_ABOUT);
	MenuElem	fAbout (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlFile.push_back (fAbout);
	mlFile.push_back (SeparatorElem());

	s=_(MENU_FILE_QUIT);
	MenuElem	fQuit (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::fileMenuCallback), string (s)));
	mlFile.push_back (fQuit);


	// 
	// Build HF menu
	//
	s=_(MENU_HF_CRATERS);
	MenuElem	hCraters (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hCraters);

	s=_(MENU_HF_FILL);
	MenuElem	hFill (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hFill);

	s=_(MENU_HF_FOLD);
	MenuElem	hFold (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hFold);

	s=_(MENU_HF_GAUSS_HILL);
	MenuElem	hGauss (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hGauss);

	s=_(MENU_HF_LINSCALE);
	MenuElem	hLinScale (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hLinScale);

	s=_(MENU_HF_MIRROR);
	MenuElem	hMirror (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hMirror);

	s=_(MENU_HF_ROTATE);
	MenuElem	hRotate (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hRotate);

	s=_(MENU_HF_ROUGHSMOOTH);
	MenuElem	hRoughSmooth (s, ACC_C, 
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hRoughSmooth);

	s=_(MENU_HF_TERRACE);
	MenuElem	hTerrace (s, ACC_C,
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hTerrace);

	s=_(MENU_HF_TRANSFORM);
	MenuElem	hTransform (s, ACC_C,
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hTransform);
	mlHF.push_back (SeparatorElem());

	s=_(MENU_HF_CLONE);
	MenuElem	hClone(s, ACC_C,
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hClone);

	s=_(MENU_HF_EXPORT_CONTOUR);
	MenuElem	hContour(s, ACC_C,
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hContour);

	s=_(MENU_HF_ERODE);
	MenuElem	hErode (s, ACC_C,
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hErode);

	s=_(MENU_HF_FLOWMAP);
	MenuElem	hFlowmap (s, ACC_C,
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hFlowmap);

	s=_(MENU_HF_INVERT);
	MenuElem	hInvert (s, ACC_C,
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hInvert);

	s=_(MENU_HF_PLACEPOV);
	MenuElem	hPPov (s, ACC_C,
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hPPov);

	s=_(MENU_HF_RESCALE);
	MenuElem	hRescale (s, ACC_C,
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hRescale);
	mlHF.push_back (SeparatorElem());

	s=_(MENU_HF_RENDER);
	MenuElem	hRender (s, ACC_C,
			SigC::bind(SigC::slot(this, &TFWindow::hfMenuCallback), string (s)));
	mlHF.push_back (hRender);


	// 
	// Build View menu
	//
	Gtk::RadioMenuItem::Group	gV;
	RadioMenuElem	v2DP (gV, _(MENU_VIEW_2DPLANE), "<control>1",
			SigC::bind(SigC::slot(this, &TFWindow::viewMenuCallback), string (_(MENU_VIEW_2DPLANE))));
	mlView.push_back (v2DP);
	RadioMenuElem	v3DW (gV, _(MENU_VIEW_3DWIRE), "<control>2",
			SigC::bind(SigC::slot(this, &TFWindow::viewMenuCallback), string (_(MENU_VIEW_3DWIRE))));
	mlView.push_back (v3DW);
	RadioMenuElem	v3DH (gV, _(MENU_VIEW_3DHEIGHT), "<control>3",
			SigC::bind(SigC::slot(this, &TFWindow::viewMenuCallback), string (_(MENU_VIEW_3DHEIGHT))));
	mlView.push_back (v3DH);
	RadioMenuElem	v3DL (gV, _(MENU_VIEW_3DLIGHT), "<control>4",
			SigC::bind(SigC::slot(this, &TFWindow::viewMenuCallback), string (_(MENU_VIEW_3DLIGHT))));
	mlView.push_back (v3DL);
	mlView.push_back (SeparatorElem());

	CheckMenuElem 	vFast (_(MENU_VIEW_FASTWIRE), "<control>5", 
			SigC::bind(SigC::slot(this, &TFWindow::viewMenuCallback), string (_(MENU_VIEW_FASTWIRE))));
	mlView.push_back (vFast);
	CheckMenuElem 	vAuto (_(MENU_VIEW_AUTOROTATE), "<control>6", 
			SigC::bind(SigC::slot(this, &TFWindow::viewMenuCallback), string (_(MENU_VIEW_AUTOROTATE))));
	mlView.push_back (vAuto);


	// 
	// Build Colormap menu
	//
	Gtk::RadioMenuItem::Group	gC;
	RadioMenuElem	cColBands (gC, _(MENU_CMAP_COLORBANDS), "<alt>1",
			SigC::bind(SigC::slot(this, &TFWindow::colormapMenuCallback), string (_(MENU_CMAP_COLORBANDS))));
	mlCMap.push_back (cColBands);
	RadioMenuElem	cDesert (gC, _(MENU_CMAP_DESERT), "<alt>2",
			SigC::bind(SigC::slot(this, &TFWindow::colormapMenuCallback), string (_(MENU_CMAP_DESERT))));
	mlCMap.push_back (cDesert);
	RadioMenuElem	cGray (gC, _(MENU_CMAP_GRAYSCALE), "<alt>3",
			SigC::bind(SigC::slot(this, &TFWindow::colormapMenuCallback), string (_(MENU_CMAP_GRAYSCALE))));
	mlCMap.push_back (cGray);
	RadioMenuElem	cRedHot (gC, _(MENU_CMAP_REDHOT), "<alt>4",
			SigC::bind(SigC::slot(this, &TFWindow::colormapMenuCallback), string (_(MENU_CMAP_REDHOT))));
	mlCMap.push_back (cRedHot);


	// Build main menu structure
	s=_(MENU_FILE_BRANCH);
	MenuElem	f (s, ACC_A, *mFile);			// File
	s=_(MENU_HF_BRANCH);
	MenuElem	h (s, ACC_A, *mHF);			// HF
	s=_(MENU_VIEW_BRANCH);
	MenuElem	v (s, ACC_A, *mView);			// View
	s=_(MENU_CMAP_BRANCH);
	MenuElem	c (s, ACC_A, *mCMap);			// Colormap

	if (TFOptions::s_fixedMenus)
		{
		p_menuBar->items().push_back (f);
		p_menuBar->items().push_back (h);
		p_menuBar->items().push_back (v);
		p_menuBar->items().push_back (c);
		p_win->add_accel_group (*accgrp);
		}
	else
		{
		Gtk::Menu		*mPopup = manage (new Gtk::Menu());
		MenuList 		mlPopup = mPopup->items();

		mlPopup.push_back (f);
		mlPopup.push_back (h);
		mlPopup.push_back (v);
		mlPopup.push_back (c);

		mPopup->accelerate (*p_win);
		p_popupMenu = mPopup;
		}
		

	//defaultView = p_itemFactory->get_checkmenuitem_widget (defaultKey);
	//defaultView->set_active (TRUE);
	delete [] t;
	}
}


/*
 *  allocColormaps: allocate the colormaps 
 */
void TFWindow::allocColormaps ()
{
	// create colormaps
	p_linearColorMap = new GuiColormapLinear (this->get_toplevel(), 255);
	p_linearColorMap->allocColors (225, 0, 0, 0, 1, 1, 1);

	p_linearColorMap2 = new GuiColormapLinear (this->get_toplevel(), 255);
	p_linearColorMap2->allocColors (225, 0, 1, 0, 1, 0, 0);

	p_bandColorMap = new GuiColormapBands (this->get_toplevel(), 15, 15, 4);
	p_bandColorMap->allocColorBands (bandBaseCol); 

	p_bandColorMap2 = new GuiColormapBands (this->get_toplevel(), 6, 15, 1);
	p_bandColorMap2->allocColorBands (bandBaseCol_Desert); 

	d_useLinearColormap = FALSE;
}


/*
 *  fileMenuCallback: default file menu handler
 */
void TFWindow::fileMenuCallback(string m)
{
	if (m == "Exit")
		quit ();
	//cout << "Callback " << m << " caught " << endl;
}


/*
 *  hfMenuCallback: default HF menu handler
 */
void TFWindow::hfMenuCallback(string m)
{
	//cout << "Callback " << m << " caught " << endl;
}


/*
 *  viewMenuCallback: default view menu handler
 */
void TFWindow::viewMenuCallback(string m)
{
	//cout << "Callback " << m << " caught " << endl;
}


/*
 *  colormapMenuCallback: default colormap menu handler
 */
void TFWindow::colormapMenuCallback(string m)
{
	//cout << "Callback " << m << " caught " << endl;
}



/*
 *  setSize: set the size of all the relevant widgets depending on the 
 * 	menu mode used. 
 */
void TFWindow::setSize (int x, int y)
{
	if ((this->width() != x) || (this->height() != y))
		{
		if (TFOptions::s_fixedMenus)
			p_win->set_usize (x, y+p_menuBar->height());
		else
			p_win->set_usize (x, y);
		this->setUsize (x, y);

		p_win->show ();
		this->show();
		gdk_flush ();

		Gtk::Main *gtkMain = Gtk::Main::instance();
		while (gtkMain->events_pending())
			gtkMain->iteration ();
		}
}


/*
 *  buildAccel: the a menu name and extract the accerlator key by 
 * 	searching for the underscore (_) marker. 
 * 
 * 	accelType denotes the mod key used: 
 *		'a'		ALT
 *		'c'		CNTRL
 * 	MenuDefs.h defines these as ACCEL_TYPE_*
 */
char *TFWindow::buildAccel (char *menu, char accelType, char *s)
{
	SanityCheck::bailout ((!menu), "menu==NULL", "TFWindow::buildAccel");
	SanityCheck::bailout ((!s), "s==NULL", "TFWindow::buildAccel");

	//if (!accelType)
	//	return NULL;

	char 	*accelKey = menu;
	char	*accel = s;

	while (*accelKey)
		{
		if (*accelKey == '_')
			{
			accelKey++;		// advance to accelrator key
			break;
			}
		accelKey++;
		}

	if (! (*accelKey))			// no accelerator found
		{
		//printf ("Menu = %s,   Accel = NULL\n", menu); fflush (stdout);
		return NULL;
		}

	strcpy (accel, "");
	if (accelType == 'a')
		sprintf (accel, "<alt>%c", toupper(*accelKey));
	else
	if (accelType == 'c')
		sprintf (accel, "<control>%c", toupper(*accelKey));
	else
		SanityCheck::bailout ((TRUE), "accelType out of Range", "TFWindow::buildAccel");

	//printf ("Menu = %s,   Accel = %s\n", menu, accel); fflush (stdout);
	if (strlen(accel))
		return accel;
	else	
		return NULL;
}


guint TFWhash (gconstpointer key)
{
	char 	*s = (char *)key;
	int	i, v=0,
		l=strlen(s);

	for (i=0; i<l; i++)
		v = (64*v + s[i])%101;

	return (v);
}


gint TFWhashcmp (gconstpointer a, gconstpointer b)
{
	return (!strcmp((char *)a,(char *)b));
}


